--
-- PipeUnloading
-- Specialization for manual PipeUnloading
--
-- @author  	gotchTOM
-- @date  		12-Aug-2018
-- @version		2.6
--
-- Fix, particles will activate if your "tipping" to ground Line 349-367 (Credits: Xentro)
-- Issue: Overloading.lua, line 238, Patch 1.3.1.0

PipeUnloading = {};

function PipeUnloading:load(savegame)

	local isPipeUnloader = false;
	local fillTypeCategory = "";
	local fillTypeName = "";
	local overloadingCapacity = 0;
	local rootName = getXMLRootName(self.xmlFile)
	local i=0;
  while true do
    local key = string.format(rootName..".fillUnits.fillUnit(%d)", i);
    if not hasXMLProperty(self.xmlFile, key) then
        break;
    end;
		local fillTypeCategories = Utils.splitString(" ", Utils.getNoNil(getXMLString(self.xmlFile, key .. "#fillTypeCategories"),""));
		for _,category in pairs(fillTypeCategories) do
			if category == "combine" or category == "augerWagon" then
				isPipeUnloader = true;
				fillTypeCategory = category;
				break;
			end;
		end;
		i=i+1;
	end;
	if self.fillUnits ~= nil and not isPipeUnloader then
		for _,fillUnit in pairs(self.fillUnits) do
			for k,fillTypes in pairs(fillUnit) do
				if k == "fillTypes" then
					for fillType,_ in pairs(fillTypes) do
						if fillType == FillUtil.FILLTYPE_SUGARBEET or fillType == FillUtil.FILLTYPE_POTATO or fillType == FillUtil.FILLTYPE_FERTILIZER or fillType == FillUtil.FILLTYPE_SEEDS or fillType == FillUtil.FILLTYPE_STRAWPELLETS or fillType == FillUtil.FILLTYPE_HAYPELLETS then
							isPipeUnloader = true;
							fillTypeName = FillUtil.fillTypeIntToName[fillType];
							break;
						end;
					end;
				end;
			end;
		end;
	end;
	if isPipeUnloader then
		isPipeUnloader = self.canUnload == nil and self.unloadingStarted == nil; -- Manuelles Abbunkern von Thomas Heck (Patar)
		if not isPipeUnloader then
			print("Manuelles Abbunkern von Thomas Heck (Patar) is in use! PipeUnloading of CombineAddOn can not be used.")
			return;
		end;
	else
		print("has not the right fillTypeCategory/fillType! PipeUnloading of CombineAddOn can not be used.")
	end;
	if self.overloading ~= nil and self.overloading.capacity ~= nil then
		overloadingCapacity = self.overloading.capacity;
	end;
	self.isPipeUnloader = overloadingCapacity > 0 and isPipeUnloader;
	if self.isPipeUnloader then
		self.autoPipeUnloading = false;
		self.isPipeUnloadingAllowed = false;
		self.puDischargeToGround = false;
		self.targetFound = false;
		self.setIsPipeUnloadingAllowed = PipeUnloading.setIsPipeUnloadingAllowed;
		self.setUnloadingMode = PipeUnloading.setUnloadingMode;
		self.findTrailerRaycastCallback = Utils.overwrittenFunction(self.findTrailerRaycastCallback, PipeUnloading.findTrailerRaycastCallback);
		self.getIsOverloadingAllowed = Utils.overwrittenFunction(self.getIsOverloadingAllowed, PipeUnloading.getIsOverloadingAllowed);
		self.onStopAiVehicle = Utils.overwrittenFunction(self.onStopAiVehicle, PipeUnloading.onStopAiVehicle);
		if fillTypeCategory ~= "" then
			print("PipeUnloading -> fillTypeCategory: "..tostring(fillTypeCategory)..", overloadingCapacity: "..tostring(overloadingCapacity));
		else
			print("PipeUnloading -> fillType: "..tostring(fillTypeName)..", overloadingCapacity: "..tostring(overloadingCapacity));
		end;
	end;
end;

function PipeUnloading:delete()
end;

function PipeUnloading:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PipeUnloading:keyEvent(unicode, sym, modifier, isDown)
end;

function PipeUnloading:update(dt)
	if self:getIsActive() and self.isPipeUnloader then
		if self.isClient and self:getIsActiveForInput(false) then
			if InputBinding.hasEvent(InputBinding.UNLOADING_MODE) then
				self:setUnloadingMode(not self.autoPipeUnloading);
			end;
			if not self.autoPipeUnloading then
				local helperIsActive = false;
				local rootVehicle = self:getRootAttacherVehicle();
				if self.cp ~= nil then
					if rootVehicle.cp.isDriving ~= nil then
						helperIsActive = rootVehicle.cp.isDriving
					end;
				end;
				if not helperIsActive and rootVehicle.isHired ~= nil then
					helperIsActive = rootVehicle.isHired;
				end;
				if not helperIsActive and (self.targetTrailer ~= nil or self.shovelTargetFound ~= nil) then
					if InputBinding.hasEvent(InputBinding.EMPTY_GRAINTANK) then
						if self.isPipeUnloadingAllowed then
							self:setIsPipeUnloadingAllowed(false);
						else
							if rootVehicle.isMotorStarted then
								self:setIsPipeUnloadingAllowed(true);
							else
								local warning = self:getTurnedOnNotAllowedWarning();
								if warning ~= nil then
									g_currentMission:showBlinkingWarning(warning, 2000);
								end
							end;
						end;
					end;
				end;
			end;
		end;
	end;
end;

function PipeUnloading:updateTick(dt)
	if self:getIsActive() and self.isPipeUnloader and not self.autoPipeUnloading then
		local helperIsActive = false;
		local cpIsDriving = false;
		if self.cp ~= nil then
			local rootVehicle = self:getRootAttacherVehicle();
			if rootVehicle.cp.isDriving ~= nil then
				cpIsDriving = rootVehicle.cp.isDriving
			end;
		end;
		if self.isHired or cpIsDriving then
			helperIsActive = true;
			if not self:getIsOverloadingAllowed() and self.overloading.isActive then
				self:setOverloadingActive(false);
			end;
		end;
		if not helperIsActive then
			if self.isServer then
				if self.pipeStateIsUnloading[self.pipeCurrentState] and self:getUnitFillLevel(self.overloading.fillUnitIndex) > 0 then
					if self.targetTrailer ~= nil then
						if self.isPipeUnloadingAllowed then
							local unloadFillType = 0;
							local trailerFillTypes = self.targetTrailer:getCurrentFillTypes()
							local fillTypes = self:getCurrentFillTypes()
							for k,fillType in pairs (fillTypes) do
								for k,trailerFillType in pairs (trailerFillTypes) do
									if trailerFillType == fillType then
										unloadFillType = trailerFillType;
										break;
									end;
								end;
							end;
							if unloadFillType > 0 then
								local fillLevel = self.targetTrailer:getFillLevel(unloadFillType)
								local capacity = self.targetTrailer:getCapacity(unloadFillType)
								if fillLevel == capacity then
									self:setIsPipeUnloadingAllowed(false);
									return;
								end;
							end;
							if self.trailerFound ~= 0 then
								if self.puDischargeToGround then
									self.puDischargeToGround = false;
								end;
							else --self.trailerFound == 0
								if self:getCanTipToGround() then
									if not self.puDischargeToGround then
										self:setDischargeToGround(true)
										self.puDischargeToGround = true;
									end;
								else
									if self.puDischargeToGround then
										self:setDischargeToGround(false)
										self.puDischargeToGround = false;
									end;
								end;
							end;
						else -- not self.isPipeUnloadingAllowed
							if self.puDischargeToGround then
								self:setDischargeToGround(false)
								self.puDischargeToGround = false;
							end;
						end;
					else --self.targetTrailer == nil
						if self.shovelTargetFound == nil then
							if self.isPipeUnloadingAllowed then
								self:setIsPipeUnloadingAllowed(false);
							end;
							if self.puDischargeToGround then
								self:setDischargeToGround(false)
								self.puDischargeToGround = false;
							end;
						end;
					end;
					if self.targetFound then
						if self.trailerFound == 0 and self.shovelTargetFound == nil then
							self.targetFound = false;
							g_server:broadcastEvent(TargetFoundEvent:new(self, self.targetFound), nil, nil, self);
						end;
					else
						if self.shovelTargetFound ~= nil or self.trailerFound ~= 0 then
							self.targetFound = true;
							g_server:broadcastEvent(TargetFoundEvent:new(self, self.targetFound), nil, nil, self);
						end;
					end;
				else --pipe or full
					if self.isPipeUnloadingAllowed then
						self:setIsPipeUnloadingAllowed(false);
					end;
					if self.puDischargeToGround then
						self:setDischargeToGround(false)
						self.puDischargeToGround = false;
					end;
				end;
			end;
			if self.isClient then
				if self.targetTrailer == nil and self.overloading ~= nil then
					if self.overloading.didOverload then
						self.overloading.didOverload = false;
					end;
				end;
			end;
		else -- helperIsActive
			if not self.isPipeUnloadingAllowed then
				self:setIsPipeUnloadingAllowed(true);
			end;
		end;
	end;
end;

function PipeUnloading:draw()
	if self.isClient and self:getIsActiveForInput(false) and self.isPipeUnloader then
		if not self.autoPipeUnloading then
			if self.hasPipe and (self.targetTrailer ~= nil or self.shovelTargetFound ~= nil) then
				if self.pipeStateIsUnloading[self.pipeCurrentState] then
					if self.isPipeUnloadingAllowed then
						g_currentMission:addHelpButtonText(g_i18n:getText("EMPTY_GRAINTANK_on"), InputBinding.EMPTY_GRAINTANK, nil, GS_PRIO_HIGH);
					elseif not self.isPipeUnloadingAllowed then
						if self.targetFound then
							g_currentMission:addHelpButtonText(g_i18n:getText("EMPTY_GRAINTANK_off"), InputBinding.EMPTY_GRAINTANK, nil, GS_PRIO_HIGH);
						else
							g_currentMission:addHelpButtonText(g_i18n:getText("EMPTY_GRAINTANK_off").."!", InputBinding.EMPTY_GRAINTANK, nil, GS_PRIO_HIGH);
						end;
					end;
				end;
			end;
			g_currentMission:addHelpButtonText(g_i18n:getText("UNLOADING_MODE_MANUAL"), InputBinding.UNLOADING_MODE, nil, GS_PRIO_HIGH);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("UNLOADING_MODE_AUTO"), InputBinding.UNLOADING_MODE, nil, GS_PRIO_HIGH);
		end;
  end;
end;

function PipeUnloading:setIsPipeUnloadingAllowed(isPipeUnloadingAllowed, noEventSend)
	if isPipeUnloadingAllowed ~= self.isPipeUnloadingAllowed then
		SetIsPipeUnloadingAllowedEvent.sendEvent(self, isPipeUnloadingAllowed, noEventSend);
		self.isPipeUnloadingAllowed = isPipeUnloadingAllowed;
		if not self.isPipeUnloadingAllowed then
			if self:getOverloadingActive() then
				self:setOverloadingActive(false);
			end;
		-- elseif self.isPipeUnloadingAllowed then
			-- if not self:getOverloadingActive() then
				-- self:setOverloadingActive(true);
			-- end;
		end;
	end;
end;

function PipeUnloading:setUnloadingMode(autoPipeUnloading, noEventSend)
	if autoPipeUnloading ~= self.autoPipeUnloading then
		SetUnloadingModeEvent.sendEvent(self, autoPipeUnloading, noEventSend);
		self.autoPipeUnloading = autoPipeUnloading;
		if self.autoPipeUnloading then
			if not self.isPipeUnloadingAllowed then
				self:setIsPipeUnloadingAllowed(true);
			end;
		else
			if self.isPipeUnloadingAllowed then
				self:setIsPipeUnloadingAllowed(false);
			end;
		end;
	end;
end;




-- fix Kuhn DLC DISCOLANDER XM 52
function PipeUnloading:findTrailerRaycastCallback(superFunc, transformId, x, y, z, distance)
	if superFunc ~= nil then
			superFunc(self)
	end;

	local vehicle = g_currentMission.nodeToVehicle[transformId];
  if vehicle ~= nil and vehicle ~= self then
    if self.trailerFound ~= 0 and self.trailerFoundDistance ~= 0 then
      return false;
    end;
    self.pipeFoundTrailer = true;
    if vehicle.exactFillRootNode == transformId then
      local fillType = self:getUnitLastValidFillType(self.overloading.fillUnitIndex);
      if vehicle:allowFillType(fillType) and vehicle.exactFillRootNode ~= nil then
        if vehicle:getFillLevel(fillType) < vehicle:getCapacity(fillType) then
          self.trailerFound = transformId;
					if vehicle.typeName == "pdlc_kuhnPack.sowingMachineCultivator" then
						self.trailerFoundDistance = distance+self.pipeParticleSystemExtraDistanceOnTrailer;
					end;
        end;
      end;
    else
      self.trailerFoundDistance = distance+self.pipeParticleSystemExtraDistanceOnTrailer;
    end;
  else
    local object = g_currentMission:getNodeObject(transformId);
    if object ~= nil and object.parent ~= nil and object.parent.getAllowFillTypeFromTool ~= nil then
      if object.parent:getAllowFillTypeFromTool(self:getUnitLastValidFillType(self.overloading.fillUnitIndex), TipTrigger.TOOL_TYPE_PIPE) then
        self.shovelTargetFound = object;
        self.trailerFoundDistance = distance+self.pipeParticleSystemExtraDistanceOnTrailer;
      end;
    end;
  end;
  return true;
end;

function PipeUnloading:getIsOverloadingAllowed(superFunc)
	if not self.autoPipeUnloading then
		if not self.isHired then
			if not self.isPipeUnloadingAllowed then
				return false;
			end;
		else
			if self.trailerFound == 0 then
				return false;
			end;
		end;
	end;
	if not self.dischargeToGround and self.trailerFound == 0 and self.shovelTargetFound == nil then
    return false
  end
	if superFunc ~= nil then
			superFunc(self)
	end;
	return true;
end;

function PipeUnloading:onStopAiVehicle(superFunc)
	if superFunc ~= nil then
			superFunc(self)
	end;
	if not self.autoPipeUnloading then
		if self.isPipeUnloadingAllowed then
			self.isPipeUnloadingAllowed = false;
		end;
	end;
end;

-- Fix, particles will activate if your "tipping" to ground
-- Issue: Overloading.lua, line 238, Patch 1.3.1.0
function PipeUnloading:newUpdateTick(oldFunc, dt)
	oldFunc(self, dt);

	if self.sentOurPipeFixUpdate == nil then
		self.sentOurPipeFixUpdate = false;
	end;

	if self.isServer then
		if self.targetTrailer == nil and self.shovelTargetFound == nil and self.dischargeToGround then
			if self.sentOurPipeFixUpdate ~= self.overloading.didOverload then
				self.sentOurPipeFixUpdate = self.overloading.didOverload;
				self:raiseDirtyFlags(self.overloadingDirtyFlag);
			end;
		end;
	end;
end;
Pipe.updateTick = Utils.overwrittenFunction(Pipe.updateTick, PipeUnloading.newUpdateTick);

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
SetIsPipeUnloadingAllowedEvent = {};
SetIsPipeUnloadingAllowedEvent_mt = Class(SetIsPipeUnloadingAllowedEvent, Event);

InitEventClass(SetIsPipeUnloadingAllowedEvent, "SetIsPipeUnloadingAllowedEvent");

function SetIsPipeUnloadingAllowedEvent:emptyNew()
  local self = Event:new(SetIsPipeUnloadingAllowedEvent_mt);
  return self;
end;

function SetIsPipeUnloadingAllowedEvent:new(vehicle, isPipeUnloadingAllowed)
  local self = SetIsPipeUnloadingAllowedEvent:emptyNew()
  self.vehicle = vehicle;
	self.isPipeUnloadingAllowed = isPipeUnloadingAllowed;
  return self;
end;

function SetIsPipeUnloadingAllowedEvent:readStream(streamId, connection)
	self.vehicle = readNetworkNodeObject(streamId);
	self.isPipeUnloadingAllowed = streamReadBool(streamId);
  self:run(connection);
end;

function SetIsPipeUnloadingAllowedEvent:writeStream(streamId, connection)
  writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteBool(streamId, self.isPipeUnloadingAllowed);
end;

function SetIsPipeUnloadingAllowedEvent:run(connection)
	self.vehicle:setIsPipeUnloadingAllowed(self.isPipeUnloadingAllowed, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(SetIsPipeUnloadingAllowedEvent:new(self.vehicle, self.isPipeUnloadingAllowed), nil, connection, self.vehicle);
	end;
end;

function SetIsPipeUnloadingAllowedEvent.sendEvent(vehicle, isPipeUnloadingAllowed, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetIsPipeUnloadingAllowedEvent:new(vehicle, isPipeUnloadingAllowed), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetIsPipeUnloadingAllowedEvent:new(vehicle, isPipeUnloadingAllowed));
		end;
	end;
end;


------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
SetUnloadingModeEvent = {};
SetUnloadingModeEvent_mt = Class(SetUnloadingModeEvent, Event);

InitEventClass(SetUnloadingModeEvent, "SetUnloadingModeEvent");

function SetUnloadingModeEvent:emptyNew()
  local self = Event:new(SetUnloadingModeEvent_mt);
  return self;
end;

function SetUnloadingModeEvent:new(vehicle, autoPipeUnloading)
  local self = SetUnloadingModeEvent:emptyNew()
  self.vehicle = vehicle;
	self.autoPipeUnloading = autoPipeUnloading;
  return self;
end;

function SetUnloadingModeEvent:readStream(streamId, connection)
	self.vehicle = readNetworkNodeObject(streamId);
	self.autoPipeUnloading = streamReadBool(streamId);
  self:run(connection);
end;

function SetUnloadingModeEvent:writeStream(streamId, connection)
  writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteBool(streamId, self.autoPipeUnloading);
end;

function SetUnloadingModeEvent:run(connection)
	self.vehicle:setUnloadingMode(self.autoPipeUnloading, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(SetUnloadingModeEvent:new(self.vehicle, self.autoPipeUnloading), nil, connection, self.vehicle);
	end;
end;

function SetUnloadingModeEvent.sendEvent(vehicle, autoPipeUnloading, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetUnloadingModeEvent:new(vehicle, autoPipeUnloading), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetUnloadingModeEvent:new(vehicle, autoPipeUnloading));
		end;
	end;
end;

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
TargetFoundEvent = {};
TargetFoundEvent_mt = Class(TargetFoundEvent, Event);

InitEventClass(TargetFoundEvent, "TargetFoundEvent");

function TargetFoundEvent:emptyNew()
  local self = Event:new(TargetFoundEvent_mt);
  return self;
end;

function TargetFoundEvent:new(object, targetFound)
  local self = TargetFoundEvent:emptyNew()
  self.object = object;
	self.targetFound = targetFound;
  return self;
end;

function TargetFoundEvent:readStream(streamId, connection)
	self.object = readNetworkNodeObject(streamId);
	self.targetFound = streamReadBool(streamId);
  self:run(connection);
end;

function TargetFoundEvent:writeStream(streamId, connection)
  writeNetworkNodeObject(streamId, self.object);
	streamWriteBool(streamId, self.targetFound);
end;

function TargetFoundEvent:run(connection)
	self.object.targetFound = self.targetFound
end;