--
-- ThreshingCounter
-- Specialization for counting threshed hectars
--
-- @author:  	Manuel Leithner (edit by gotchTOM)
-- @date:			26-Feb-2017
-- @version:	v4.0.1
-- @history:	v2.0.0 - initial implementation
--						v2.1.0 - ready for "BasisMOD" (move hudTipperOverlay, hudAttachmentOverlay, hudFuelOverlay)
--						v2.2.0 - reset sessionHectars
--						v2.3.0 - correct rounding
--						v2.4.0 - ha/h
--						v3.0.0 - convert to LS15
--						v3.1.0 - fix background overlay
--						v3.2.0 - fix synchronisation
--						v3.6.0 - fix HUD
--						v3.7.0 - load/save workedHectars
--						v3.8.0 - Eintrag für Gezogene Roder und Häcksler gelöscht, Bug wurde von Giants behoben, dadurch gab es einen doppelten Eintrag
--						v3.9.0 - convert to FS17
--						v4.0.0 - fix: save workedHectars
--						v4.0.1 - fix: get self.workedHectarsBackup
--
-- free for noncommerical-usage
--

ThreshingCounter = {};

local mod_directory = g_currentModDirectory;

function ThreshingCounter.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Combine, specializations);
end;

function ThreshingCounter:load(savegame)
	self.resetSessionHectars = SpecializationUtil.callSpecializationsFunction("resetSessionHectars");
	self.workedHectarsBackup = 0;
	self.sessionHectars = 0;
	self.hectarPerHour = 0;
	self.hectarTimer = 0;
	self.sessionHectarsSent = 0;
	self.threshingCounterDirtyFlag = self:getNextDirtyFlag();
	self.ThreshingCounter = {};
	self.ThreshingCounter.maxHUDposition = 2;
	self.ThreshingCounter.lastNumActiveHUDs = 0;
	self.ThreshingCounter.totalHUD = {};
	self.ThreshingCounter.sessionHUD = {};
	local xOffsetIcon = g_currentMission.vehicleHudBg.width / 12;
	self.ThreshingCounter.totalHUD.xPos = g_currentMission.vehicleHudBg.x + xOffsetIcon;
	self.ThreshingCounter.sessionHUD.xPos = g_currentMission.vehicleHudBg.x + g_currentMission.vehicleHudBg.width/2.1 + xOffsetIcon;
	self.ThreshingCounter.yPos = g_currentMission.vehicleHudBg.y + g_currentMission.vehicleHudBg.height;
	self.ThreshingCounter.yOffsetIcon = g_currentMission.fuelLevelIconOverlay.height * 0.14;
	self.ThreshingCounter.overlayHeight = g_currentMission.fuelLevelIconOverlay.height;
	self.ThreshingCounter.overlayWidth = self.ThreshingCounter.overlayHeight / g_screenAspectRatio;
	self.ThreshingCounter.bgOverlay = Overlay:new("hudThreshingCounterBackgroundOverlay", Utils.getFilename("ThreshingCounter/hud_bg.dds", mod_directory), g_currentMission.vehicleHudBg.x, self.ThreshingCounter.yPos, g_currentMission.vehicleHudBg.width, g_currentMission.fuelLevelIconOverlay.height);
	self.ThreshingCounter.totalHUD.overlay = Overlay:new("hudThreshingCounterOverlay", Utils.getFilename("ThreshingCounter/ThreshingCounter_totalHUD.dds", mod_directory), self.ThreshingCounter.totalHUD.xPos, self.ThreshingCounter.yPos + self.ThreshingCounter.yOffsetIcon, self.ThreshingCounter.overlayWidth, self.ThreshingCounter.overlayHeight);
	self.ThreshingCounter.sessionHUD.overlay = Overlay:new("hudThreshingCounterOverlay", Utils.getFilename("ThreshingCounter/ThreshingCounter_sessionHUD.dds", mod_directory), self.ThreshingCounter.sessionHUD.xPos, self.ThreshingCounter.yPos + self.ThreshingCounter.yOffsetIcon, self.ThreshingCounter.overlayWidth, self.ThreshingCounter.overlayHeight);
	self.ThreshingCounter.textSize = g_currentMission.operatingTimeTextSize;
end;

function ThreshingCounter:postLoad(savegame)
	if savegame ~= nil then
		self.workedHectarsBackup = Utils.getNoNil(getXMLFloat(savegame.xmlFile, savegame.key .. "#workedHectars"), self.workedHectars);
	end;
end;

function ThreshingCounter:delete()
end;

function ThreshingCounter:readStream(streamId, connection)
	local workedBackup = streamReadFloat32(streamId);
	local worked = streamReadFloat32(streamId);
	local session = streamReadFloat32(streamId);
	local hectarTimer = streamReadFloat32(streamId);
	self.workedHectarsBackup = workedBackup;
	self.workedHectars = worked;
	self.sessionHectars = session;
	self.hectarTimer = hectarTimer;
end;

function ThreshingCounter:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self.workedHectarsBackup);
	streamWriteFloat32(streamId, self.workedHectars);
	streamWriteFloat32(streamId, self.sessionHectars);
	streamWriteFloat32(streamId, self.hectarTimer);
end;

function ThreshingCounter:readUpdateStream(streamId, timestamp, connection)
  if connection:getIsServer() then
    if streamReadBool(streamId) then
			self.sessionHectars = streamReadFloat32(streamId);
			self.workedHectars = streamReadFloat32(streamId);
    end;
  end;
end;

function ThreshingCounter:writeUpdateStream(streamId, connection, dirtyMask)
  if not connection:getIsServer() then
    if streamWriteBool(streamId, bitAND(dirtyMask, self.threshingCounterDirtyFlag) ~= 0) then
			streamWriteFloat32(streamId, self.sessionHectars);
			streamWriteFloat32(streamId, self.workedHectars);
    end;
  end;
end;

function ThreshingCounter:getSaveAttributesAndNodes(nodeIdent)
	local attributes = "";
	if not self.chopperSwitchAvailable then
		attributes = 'workedHectars="' .. tostring(self.workedHectars) ..'"';
	end;
	return attributes, nil;
end;

function ThreshingCounter:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ThreshingCounter:keyEvent(unicode, sym, modifier, isDown)
end;

function ThreshingCounter:update(dt)
	if self:getIsActiveForInput(false) then
		if InputBinding.hasEvent(InputBinding.RESET_SESSION_HA) then
			self:resetSessionHectars();
		end;
	end;
end;

function ThreshingCounter:updateTick(dt)
	if self.turnOnVehicle ~= nil and self.turnOnVehicle.isTurnedOn then
		self.hectarTimer = self.hectarTimer + dt;
		local hectarTimer = self.hectarTimer/3600000
		self.hectarPerHour = self.sessionHectars/hectarTimer;
		self.sessionHectars = self.workedHectars - self.workedHectarsBackup;
		if math.abs(self.sessionHectars - self.sessionHectarsSent) > 0.01 then
      self:raiseDirtyFlags(self.threshingCounterDirtyFlag);
			self.sessionHectarsSent = self.sessionHectars;
    end;
	end;
	if self:getIsActive() then
		if self.AttacherVehicleBackup == nil then
			local attacherVehicle = self:getRootAttacherVehicle();
			self.AttacherVehicleBackup = attacherVehicle;
		end;
		if self.AttacherVehicleBackup.ActiveHUDs == nil then
			self.AttacherVehicleBackup.ActiveHUDs = {};
			self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs = 0;
		end;
		if self.AttacherVehicleBackup.ActiveHUDs ~= nil then
			if self.AttacherVehicleBackup.ActiveHUDs.threshingCounter == nil then
				self.AttacherVehicleBackup.ActiveHUDs.threshingCounter = true;
				self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs = self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs + 1;
				self.ThreshingCounter.lastNumActiveHUDs = self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs;

				self.ThreshingCounter.yPos = g_currentMission.vehicleHudBg.y + g_currentMission.vehicleHudBg.height - g_currentMission.fuelLevelIconOverlay.height + g_currentMission.fuelLevelIconOverlay.height*math.min(self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs, self.ThreshingCounter.maxHUDposition);

				self.ThreshingCounter.bgOverlay = Overlay:new("hudThreshingCounterBackgroundOverlay", Utils.getFilename("ThreshingCounter/hud_bg.dds", mod_directory), g_currentMission.vehicleHudBg.x, self.ThreshingCounter.yPos, g_currentMission.vehicleHudBg.width, g_currentMission.fuelLevelIconOverlay.height + 2*self.ThreshingCounter.yOffsetIcon);

				self.ThreshingCounter.totalHUD.overlay = Overlay:new("hudThreshingCounterOverlay", Utils.getFilename("ThreshingCounter/ThreshingCounter_totalHUD.dds", mod_directory), self.ThreshingCounter.totalHUD.xPos, self.ThreshingCounter.yPos + self.ThreshingCounter.yOffsetIcon, self.ThreshingCounter.overlayWidth, self.ThreshingCounter.overlayHeight);

				self.ThreshingCounter.sessionHUD.overlay = Overlay:new("hudThreshingCounterOverlay", Utils.getFilename("ThreshingCounter/ThreshingCounter_sessionHUD.dds", mod_directory), self.ThreshingCounter.sessionHUD.xPos, self.ThreshingCounter.yPos + self.ThreshingCounter.yOffsetIcon, self.ThreshingCounter.overlayWidth, self.ThreshingCounter.overlayHeight);
			else
				if self.ThreshingCounter.lastNumActiveHUDs ~= self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs then
					self.ThreshingCounter.yPos = g_currentMission.vehicleHudBg.y + g_currentMission.vehicleHudBg.height - g_currentMission.fuelLevelIconOverlay.height + g_currentMission.fuelLevelIconOverlay.height*math.min(self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs, self.ThreshingCounter.maxHUDposition);

					self.ThreshingCounter.bgOverlay = Overlay:new("hudThreshingCounterBackgroundOverlay", Utils.getFilename("ThreshingCounter/hud_bg.dds", mod_directory), g_currentMission.vehicleHudBg.x, self.ThreshingCounter.yPos, g_currentMission.vehicleHudBg.width, g_currentMission.fuelLevelIconOverlay.height + 2*self.ThreshingCounter.yOffsetIcon);

					self.ThreshingCounter.totalHUD.overlay = Overlay:new("hudThreshingCounterOverlay", Utils.getFilename("ThreshingCounter/ThreshingCounter_totalHUD.dds", mod_directory), self.ThreshingCounter.totalHUD.xPos, self.ThreshingCounter.yPos + self.ThreshingCounter.yOffsetIcon, self.ThreshingCounter.overlayWidth, self.ThreshingCounter.overlayHeight);
					self.ThreshingCounter.sessionHUD.overlay = Overlay:new("hudThreshingCounterOverlay", Utils.getFilename("ThreshingCounter/ThreshingCounter_sessionHUD.dds", mod_directory), self.ThreshingCounter.sessionHUD.xPos, self.ThreshingCounter.yPos + self.ThreshingCounter.yOffsetIcon, self.ThreshingCounter.overlayWidth, self.ThreshingCounter.overlayHeight);

					self.ThreshingCounter.lastNumActiveHUDs = self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs;
				end;
			end;
		end;
	else
		if self.AttacherVehicleBackup ~= nil and self.AttacherVehicleBackup.ActiveHUDs ~= nil and self.AttacherVehicleBackup.ActiveHUDs.threshingCounter then
			self.AttacherVehicleBackup.ActiveHUDs.threshingCounter = nil;
			self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs = self.AttacherVehicleBackup.ActiveHUDs.numActiveHUDs - 1;
			self.AttacherVehicleBackup = nil;
		end;
	end;
end;

function ThreshingCounter:draw()
	if self.isClient and self:getIsActive() and self.ThreshingCounter ~= nil then
		self.ThreshingCounter.bgOverlay:render();
		self.ThreshingCounter.totalHUD.overlay:render();
		self.ThreshingCounter.sessionHUD.overlay:render();

		local counterSession = math.floor(self.sessionHectars*100 + 0.5) / 100;
		local counterTotal = math.floor(self.workedHectars*10 + 0.5) / 10;
		local fullSession = math.floor(counterSession);
		local fullTotal = math.floor(counterTotal);
		local deciSession = math.floor((counterSession - fullSession)*100);
		if deciSession < 10 then
			deciSession = "0" .. deciSession;
		end;
		local deciTotal = math.floor((counterTotal - fullTotal)*10);

		setTextAlignment(RenderText.ALIGN_RIGHT);
		setTextBold(false);

		local charWidth = g_currentMission.vehicleHudBg.width * 0.035;
		local nNumOffsetTotal = (string.len(fullTotal)+1.2)*charWidth;
		local nNumOffsetSession = (string.len(fullSession)+1.2)*charWidth;

		local xOffsetTextTotal = self.ThreshingCounter.overlayWidth + nNumOffsetTotal;
		local xOffsetTextSession = self.ThreshingCounter.overlayWidth + nNumOffsetSession;

		local yOffsetText = 2.5*self.ThreshingCounter.yOffsetIcon;

		setTextColor(1,1,1,1);
		renderText(self.ThreshingCounter.totalHUD.xPos+xOffsetTextTotal, self.ThreshingCounter.yPos + yOffsetText, self.ThreshingCounter.textSize, tostring(fullTotal) .. ",");
		renderText(self.ThreshingCounter.sessionHUD.xPos+xOffsetTextSession, self.ThreshingCounter.yPos + yOffsetText, self.ThreshingCounter.textSize, tostring(fullSession) .. ",");

		setTextColor(0.95,0,0,1);
		renderText(self.ThreshingCounter.totalHUD.xPos+xOffsetTextTotal + charWidth, self.ThreshingCounter.yPos + yOffsetText, self.ThreshingCounter.textSize, tostring(deciTotal));
		renderText(self.ThreshingCounter.sessionHUD.xPos+xOffsetTextSession + 2*charWidth, self.ThreshingCounter.yPos + yOffsetText, self.ThreshingCounter.textSize, tostring(deciSession));

		setTextColor(1,1,1,1);
		renderText(self.ThreshingCounter.totalHUD.xPos+xOffsetTextTotal + charWidth*3.14, self.ThreshingCounter.yPos + yOffsetText, self.ThreshingCounter.textSize, "ha");
		renderText(self.ThreshingCounter.sessionHUD.xPos+xOffsetTextSession + charWidth*4.14, self.ThreshingCounter.yPos + yOffsetText, self.ThreshingCounter.textSize, "ha");

		setTextAlignment(RenderText.ALIGN_LEFT);

		local haPerHour = math.floor(self.hectarPerHour*10 + 0.5) / 10;
		g_currentMission:addExtraPrintText(string.format("%.1f ha/h",haPerHour));
		g_currentMission:addHelpButtonText(g_i18n:getText("RESET_SESSION_HA"), InputBinding.RESET_SESSION_HA, nil, GS_PRIO_HIGH);
	end;
end;

function ThreshingCounter:resetSessionHectars(noEventSend)
	if noEventSend == nil or noEventSend == false then
		ResetSessionHectarsEvent.sendEvent(self, noEventSend);
	end;
	self.workedHectarsBackup = self.workedHectars
	self.hectarTimer = 0;
	self.hectarPerHour = 0;
	self.sessionHectars = self.workedHectars - self.workedHectarsBackup;
end;

--
--
--
--
ResetSessionHectarsEvent = {};
ResetSessionHectarsEvent_mt = Class(ResetSessionHectarsEvent, Event);

InitEventClass(ResetSessionHectarsEvent, "ResetSessionHectarsEvent");

function ResetSessionHectarsEvent:emptyNew()
  local self = Event:new(ResetSessionHectarsEvent_mt);
  return self;
end;

function ResetSessionHectarsEvent:new(vehicle)
  local self = ResetSessionHectarsEvent:emptyNew()
  self.vehicle = vehicle;
  return self;
end;

function ResetSessionHectarsEvent:readStream(streamId, connection)
  self.vehicle = readNetworkNodeObject(streamId);
  self:run(connection);
end;

function ResetSessionHectarsEvent:writeStream(streamId, connection)
  writeNetworkNodeObject(streamId, self.vehicle);
end;

function ResetSessionHectarsEvent:run(connection)
	if not connection:getIsServer() then
		g_server:broadcastEvent(ResetSessionHectarsEvent:new(self.vehicle), nil, connection, self.vehicle);
	end;
	if self.vehicle ~= nil then
		self.vehicle:resetSessionHectars(true);
	end;
end;

function ResetSessionHectarsEvent.sendEvent(vehicle, noEventSend)
	if g_server ~= nil then
		g_server:broadcastEvent(ResetSessionHectarsEvent:new(vehicle), nil, nil, vehicle);
	else
		g_client:getServerConnection():sendEvent(ResetSessionHectarsEvent:new(vehicle));
	end;
end;